﻿angular.module('shopping.list.helper.service', [])

.factory('ShoppingListHelper', ['$q', '$ionicLoading', 'mallistaDB', 'TranslatorService', 'ToolService',
    function ($q, $ionicLoading, mallistaDB, TranslatorService, ToolService) {
    var self = this;

    //Insert a new shopping list
    self._addList = function (name, remarks) {
        var q = $q.defer();
        var query = "INSERT INTO ShoppingLists (name, remarks) VALUES (?, ?);";

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, [name, remarks]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Delete a specific shopping list
    self._deleteList = function (id) {
        var q = $q.defer();
        var query = "UPDATE ShoppingLists SET activated = 0, modificationDate = datetime('now', 'localtime') WHERE shoppingListId = ?;";

        ToolService.showLoading("DELETING_DATA");
        mallistaDB.executeSql(query, [id]).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get a shopping list name from its identificator
    self._getShoppingListName = function (shoppingListId) {
        var q = $q.defer();
        var query = "SELECT name FROM ShoppingLists WHERE shoppingListId = ?;";
        var result = null;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [shoppingListId]).then(function (res) {
            if (res.rows.length > 0) {
                result = res.rows.item(0).name;
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Get all activated shopping lists
    self._getLists = function (allItem, fullList) {
        var q = $q.defer();
        var query = "SELECT shoppingListId AS id, activated," +
                    " strftime('%d/%m/%Y %H:%M:%S', creationDate) AS creationDate," +
                    " name, remarks" +
                    " FROM ShoppingLists" +
                    " WHERE {0}" +
                    " ORDER BY name COLLATE NOCASE ASC;";
        var result = [];
        var where = "activated = 1";

        if (allItem != undefined && allItem == true) {
            var allItems = { id: 0, name: TranslatorService.getToken("ALL_LISTS") };
            result.push(allItems);
        }

        if (fullList != undefined && fullList == true)
            where = "1 = 1";

        query = TranslatorService.format(query, where);

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query).then(function (res) {
            if (res.rows.length > 0) {
                for (var i = 0; i < res.rows.length; i++) {
                    var item = res.rows.item(i);
                    result.push(item);
                }
            }
            $ionicLoading.hide();
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Indicate whether a shopping list exists on database
    self._isRegistered = function (name) {
        var q = $q.defer();
        var query = "SELECT COUNT(*) AS total FROM ShoppingLists WHERE name LIKE ?;";
        var result = false;

        ToolService.showLoading("LOADING_DATA");
        mallistaDB.executeSql(query, [name]).then(function (res) {
            $ionicLoading.hide();
            if (res.rows.length > 0) {
                result = res.rows.item(0).total > 0;
            }
            q.resolve(result);
        }, function (err) {
            $ionicLoading.hide();
            q.reject(err);
        });
        return q.promise;
    }

    //Update a specific shoppping list
    self._updateList = function (id, name, remarks) {
        var q = $q.defer();
        var query = "UPDATE ShoppingLists SET name = ?, remarks = ?, modificationDate = datetime('now', 'localtime') WHERE shoppingListId = ?;";

        ToolService.showLoading("SAVING_DATA");
        mallistaDB.executeSql(query, [name, remarks, id], true).then(function (res) {
            $ionicLoading.hide();
            q.resolve();
        }, function (err) {
            $ionicLoading.hide();
            if (err.code == 19)
                err.message = TranslatorService.getToken("ERROR_REGISTERED", name);
            ToolService.showMessage(err.message);
            q.reject(err);
        });
        return q.promise;
    }

    //Service's public methods
    return {
        addShoppingList: function (name, remarks) {
            return self._addList(name, remarks);
        },
        deleteShoppingList: function (id){
            return self._deleteList(id);
        },
        getFullShoppingLists: function (allItem) {
            return self._getLists(allItem, true);
        },
        getShoppingListName: function (shoppingListId) {
            return self._getShoppingListName(shoppingListId);
        },
        getShoppingLists: function (allItem) {
            return self._getLists(allItem);
        },
        isRegistered: function (name) {
            return self._isRegistered(name);
        },
        updateShoppingList: function (id, name, remarks) {
            return self._updateList(id, name, remarks);
        },
    };
}])
//Filter used with list of shopping lists
.filter('shoppingListFilter', [function () {
    return function (items, expression) {
        var foundItems = [];

        if (expression != undefined) {
            expression = expression.toLowerCase();

            angular.forEach(items, function (item) {
                if (item.name != undefined) {
                    if (item.name.toLowerCase().indexOf(expression) >= 0)
                        foundItems.push(item);
                }
            });
        }
        return foundItems;
    };
}]); //end module


